@@ -21,6 +21,7 @@ module Control.Arrow.ListArrows
     , module Control.Arrow.ArrowIf
     , module Control.Arrow.ArrowIO
     , module Control.Arrow.ArrowList
+    , module Control.Arrow.ArrowNavigatableTree
     , module Control.Arrow.ArrowNF
     , module Control.Arrow.ArrowState
     , module Control.Arrow.ArrowTree
@@ -29,6 +30,8 @@ module Control.Arrow.ListArrows
     , module Control.Arrow.StateListArrow
     , module Control.Arrow.IOListArrow
     , module Control.Arrow.IOStateListArrow
+
+    , module Control.Arrow.NTreeEdit            -- extra arrows
     )
 where
 
@@ -36,6 +39,7 @@ import Control.Arrow                            -- arrow classes
 import Control.Arrow.ArrowExc
 import Control.Arrow.ArrowList
 import Control.Arrow.ArrowIf
+import Control.Arrow.ArrowNavigatableTree
 import Control.Arrow.ArrowNF
 import Control.Arrow.ArrowState
 import Control.Arrow.ArrowTree
@@ -46,4 +50,6 @@ import Control.Arrow.StateListArrow
 import Control.Arrow.IOListArrow
 import Control.Arrow.IOStateListArrow
 
+import Control.Arrow.NTreeEdit                  -- extra arrows
+
 -- ------------------------------------------------------------
