@@ -162,6 +162,10 @@ instance NFData XIOSysState             -- all fields of interest are strict
 
 data XIOSysWriter       = XIOwrt  { xioErrorStatus              :: ! Int
                                   , xioErrorMsgList             :: ! XmlTrees
+                                  , xioExpatErrors              ::   IOSArrow XmlTree XmlTree
+                                  , xioRelaxNoOfErrors          :: ! Int
+                                  , xioRelaxDefineId            :: ! Int
+                                  , xioRelaxAttrList            ::   AssocList String XmlTrees
                                   }
 
 data XIOSysEnv          = XIOEnv  { xioTraceLevel               :: ! Int
@@ -203,6 +207,8 @@ data XIOParseConfig     = XIOPcfg { xioMimeTypes                ::   MimeTypeTab
                                   , xioIgnoreNoneXmlContents    :: ! Bool
                                   , xioTagSoup                  :: ! Bool
                                   , xioTagSoupParser            ::   IOSArrow XmlTree XmlTree
+                                  , xioExpat                    :: ! Bool
+                                  , xioExpatParser              ::   IOSArrow XmlTree XmlTree
                                   }
 
 data XIOOutputConfig    = XIOOcfg { xioIndent                   :: ! Bool
@@ -224,10 +230,7 @@ data XIORelaxConfig     = XIORxc  { xioRelaxValidate            :: ! Bool
                                   , xioRelaxValidateExtRef      :: ! Bool
                                   , xioRelaxValidateInclude     :: ! Bool
                                   , xioRelaxCollectErrors       :: ! Bool
-                                  , xioRelaxNoOfErrors          :: ! Int
-                                  , xioRelaxDefineId            :: ! Int
-                                  , xioRelaxAttrList            ::   AssocList String XmlTrees
-				  , xioRelaxValidator           ::   IOSArrow XmlTree XmlTree
+                                  , xioRelaxValidator           ::   IOSArrow XmlTree XmlTree
                                   }
 
 data XIOCacheConfig     = XIOCch  { xioBinaryCompression        ::   CompressionFct
@@ -237,6 +240,7 @@ data XIOCacheConfig     = XIOCch  { xioBinaryCompression        ::   Compression
                                   , xioDocumentAge              :: ! Int
                                   , xioCache404Err              :: ! Bool
                                   , xioCacheRead                ::   String -> IOSArrow XmlTree XmlTree
+                                  , xioStrictDeserialize        :: ! Bool
                                   }
 
 type CompressionFct     = ByteString -> ByteString
@@ -278,6 +282,27 @@ theErrorMsgList                 = theSysWriter
                                     , setS = \ x s -> s { xioErrorMsgList = x }
                                     }
 
+theRelaxNoOfErrors              :: Selector XIOSysState Int
+theRelaxNoOfErrors              = theSysWriter
+                                  >>>
+                                  S { getS = xioRelaxNoOfErrors
+                                    , setS = \ x s -> s { xioRelaxNoOfErrors = x}
+                                    }
+
+theRelaxDefineId                :: Selector XIOSysState Int
+theRelaxDefineId                = theSysWriter
+                                  >>>
+                                  S { getS = xioRelaxDefineId
+                                    , setS = \ x s -> s { xioRelaxDefineId = x}
+                                    }
+
+theRelaxAttrList                :: Selector XIOSysState (AssocList String XmlTrees)
+theRelaxAttrList                = theSysWriter
+                                  >>>
+                                  S { getS = xioRelaxAttrList
+                                    , setS = \ x s -> s { xioRelaxAttrList = x}
+                                    }
+
 -- ----------------------------------------
 
 theSysEnv                       :: Selector XIOSysState XIOSysEnv
@@ -464,27 +489,6 @@ theRelaxCollectErrors           = theRelaxConfig
                                     , setS = \ x s -> s { xioRelaxCollectErrors = x}
                                     }
 
-theRelaxNoOfErrors              :: Selector XIOSysState Int
-theRelaxNoOfErrors              = theRelaxConfig
-                                  >>>
-                                  S { getS = xioRelaxNoOfErrors
-                                    , setS = \ x s -> s { xioRelaxNoOfErrors = x}
-                                    }
-
-theRelaxDefineId                :: Selector XIOSysState Int
-theRelaxDefineId                = theRelaxConfig
-                                  >>>
-                                  S { getS = xioRelaxDefineId
-                                    , setS = \ x s -> s { xioRelaxDefineId = x}
-                                    }
-
-theRelaxAttrList                :: Selector XIOSysState (AssocList String XmlTrees)
-theRelaxAttrList                = theRelaxConfig
-                                  >>>
-                                  S { getS = xioRelaxAttrList
-                                    , setS = \ x s -> s { xioRelaxAttrList = x}
-                                    }
-
 theRelaxValidator                :: Selector XIOSysState (IOSArrow XmlTree XmlTree)
 theRelaxValidator                = theRelaxConfig
                                    >>>
@@ -665,6 +669,27 @@ theTagSoupParser                = theParseConfig
                                     , setS = \ x s -> s { xioTagSoupParser = x }
                                     }
 
+theExpat                        :: Selector XIOSysState Bool
+theExpat                        = theParseConfig
+                                  >>>
+                                  S { getS = xioExpat
+                                    , setS = \ x s -> s { xioExpat = x }
+                                    }
+
+theExpatParser                  :: Selector XIOSysState (IOSArrow XmlTree XmlTree)
+theExpatParser                  = theParseConfig
+                                  >>>
+                                  S { getS = xioExpatParser
+                                    , setS = \ x s -> s { xioExpatParser = x }
+                                    }
+
+theExpatErrors                  :: Selector XIOSysState (IOSArrow XmlTree XmlTree)
+theExpatErrors                  = theSysWriter
+                                  >>>
+                                  S { getS = xioExpatErrors
+                                    , setS = \ x s -> s { xioExpatErrors = x }
+                                    }
+
 -- ----------------------------------------
 
 theCacheConfig                  :: Selector XIOSysState XIOCacheConfig
@@ -723,6 +748,13 @@ theCacheRead                    = theCacheConfig
                                     , setS = \ x s -> s { xioCacheRead = x}
                                     }
 
+theStrictDeserialize            :: Selector XIOSysState Bool
+theStrictDeserialize            = theCacheConfig
+                                  >>>
+                                  S { getS = xioStrictDeserialize
+                                    , setS = \ x s -> s { xioStrictDeserialize = x}
+                                    }
+
 -- ------------------------------------------------------------
 
 getSysVar                       :: Selector XIOSysState c -> IOStateArrow s b c
