@@ -38,14 +38,15 @@ module Text.XML.HXT.Parser.XmlParsec
     , encodingDecl
     , xread
 
-    , parseXmlAttrValue
     , parseXmlContent
     , parseXmlDocEncodingSpec
     , parseXmlDocument
     , parseXmlDTDPart
     , parseXmlEncodingSpec
     , parseXmlEntityEncodingSpec
-    , parseXmlGeneralEntityValue
+    , parseXmlEntityValueAsAttrValue
+    , parseXmlEntityValueAsContent
+
     , parseXmlPart
     , parseXmlText
 
@@ -56,103 +57,121 @@ module Text.XML.HXT.Parser.XmlParsec
     )
 where
 
-import Text.ParserCombinators.Parsec
-    ( GenParser
-    , Parser
-    , parse
-    , (<?>), (<|>)
-    , char
-    , string
-    , eof
-    , between
-    , many, many1
-    , option
-    , try
-    , unexpected
-    , getPosition
-    , getInput
-    , sourceName
-    )
-
-import Text.XML.HXT.DOM.ShowXml
-    ( xshow
-    )
+import Text.ParserCombinators.Parsec                    ( runParser
+                                                        , (<?>), (<|>)
+                                                        , char
+                                                        , string
+                                                        , eof
+                                                        , between
+                                                        , many
+                                                        , many1
+                                                        , notFollowedBy
+                                                        , option
+                                                        , try
+                                                        , unexpected
+                                                        , getPosition
+                                                        , getInput
+                                                        , sourceName
+                                                        )
 
+import Text.XML.HXT.DOM.ShowXml                         ( xshow
+                                                        )
 import Text.XML.HXT.DOM.Interface
-
-import Text.XML.HXT.DOM.XmlNode
-    ( mkElement
-    , mkAttr
-    , mkRoot
-    , mkDTDElem
-    , mkText
-    , mkCmt
-    , mkCdata
-    , mkError
-    , mkPi
-    , isText
-    , isRoot
-    , getText
-    , getChildren
-    , getAttrl
-    , getAttrName
-    , changeAttrl
-    , mergeAttrl
-    )
-
-import Text.XML.HXT.Parser.XmlCharParser
-    ( xmlChar
-    )
+import Text.XML.HXT.DOM.XmlNode                         ( mkElement'
+                                                        , mkAttr'
+                                                        , mkRoot'
+                                                        , mkDTDElem'
+                                                        , mkText'
+                                                        , mkCmt'
+                                                        , mkCdata'
+                                                        , mkError'
+                                                        , mkPi'
+                                                        , isText
+                                                        , isRoot
+                                                        , getText
+                                                        , getChildren
+                                                        , getAttrl
+                                                        , getAttrName
+                                                        , changeAttrl
+                                                        , mergeAttrl
+                                                        )
+import Text.XML.HXT.Parser.XmlCharParser                ( xmlChar
+                                                        , XParser
+                                                        , SimpleXParser
+                                                        , XPState
+                                                        , withNormNewline
+                                                        , withoutNormNewline
+                                                        )
 import qualified Text.XML.HXT.Parser.XmlTokenParser     as XT
 import qualified Text.XML.HXT.Parser.XmlDTDTokenParser  as XD
 
-import Data.Char        (toLower)
+import Control.FlatSeq
+
+import Data.Char                                        (toLower)
 import Data.Maybe
 
+-- import Debug.Trace
+
 -- ------------------------------------------------------------
 --
 -- Character Data (2.4)
 
-charData                :: GenParser Char state XmlTrees
+charData                :: XParser s XmlTrees
 charData
     = many (charData' <|> XT.referenceT)
 
-charData'               :: GenParser Char state XmlTree
+charData'               :: XParser s XmlTree
 charData'
-    = try ( do
-            t <- XT.allBut1 many1 (\ c -> not (c `elem` "<&")) "]]>"
-            return (mkText $! t)
-          )
+    =  do
+       t <- XT.allBut1 many1 (\ c -> not (c `elem` "<&")) "]]>"
+       return (mkText' t)
 
 -- ------------------------------------------------------------
 --
 -- Comments (2.5)
 
-comment         :: GenParser Char state XmlTree
-
+comment         :: XParser s XmlTree
 comment
+    = comment'' $ XT.checkString "<!--"
+
+-- the leading <! is already parsed
+
+comment'        :: XParser s XmlTree
+comment'
+    = comment'' (string "--" >> return ())
+
+comment''       :: XParser s () -> XParser s XmlTree
+comment'' op
     = ( do
-        c <- between (try $ string "<!--") (string "-->") (XT.allBut many "--")
-        return (mkCmt $! c)
+        c <- between op (string ("-->")) (XT.allBut many "--")
+        return (mkCmt' c)
       ) <?> "comment"
 
 -- ------------------------------------------------------------
 --
 -- Processing Instructions
 
-pI              :: GenParser Char state XmlTree
-pI
-    = between (try $ string "<?") (string "?>")
+pI             :: XParser s XmlTree
+pI = pI'' $ XT.checkString "<?"
+
+-- the leading < is already parsed
+
+pI'             :: XParser s XmlTree
+pI' = pI'' (char '?' >> return ())
+
+pI''             :: XParser s () -> XParser s XmlTree
+pI'' op
+    = between op (string "?>")
       ( do
         n <- pITarget
-        p <- option "" (do
-                        _ <- XT.sPace
+        p <- option "" (XT.sPace
+                        >>
                         XT.allBut many "?>"
                        )
-        return $ mkPi (mkName n) [mkAttr (mkName a_value) [mkText p]]
+        return (mkPi' (mkName n) [mkAttr' (mkName a_value) [mkText' p]])
       ) <?> "processing instruction"
       where
-      pITarget  :: GenParser Char state String
+      pITarget  :: XParser s String
       pITarget = ( do
                    n <- XT.name
                    if map toLower n == t_xml
@@ -164,28 +183,38 @@ pI
 --
 -- CDATA Sections (2.7)
 
-cDSect          :: GenParser Char state XmlTree
-
+cDSect          :: XParser s XmlTree
 cDSect
+    = cDSect'' $ XT.checkString "<![CDATA["
+
+-- the leading <! is already parsed, no try neccessary
+
+cDSect'         :: XParser s XmlTree
+cDSect' 
+    = cDSect'' (string "[CDATA[" >> return ())
+
+cDSect''        :: XParser s () -> XParser s XmlTree
+cDSect'' op
     = do
-      t <- between ( try $ string "<![CDATA[") (string "]]>") (XT.allBut many "]]>")
-      return (mkCdata $! t)
+      t <- between op (string "]]>") (XT.allBut many "]]>")
+      return (mkCdata' t)
       <?> "CDATA section"
 
 -- ------------------------------------------------------------
 --
 -- Document (2.1) and Prolog (2.8)
 
-document        :: GenParser Char state XmlTree
+document        :: XParser s XmlTree
 document
     = do
       pos <- getPosition
       dl <- document'
-      return $ mkRoot [ mkAttr (mkName a_source) [mkText (sourceName pos)]
-                      , mkAttr (mkName a_status) [mkText (show c_ok)]
+      return (mkRoot' [ mkAttr' (mkName a_source) [mkText' (sourceName pos)]
+                      , mkAttr' (mkName a_status) [mkText' (show c_ok)]
                       ] dl
+             )
 
-document'       :: GenParser Char state XmlTrees
+document'       :: XParser s XmlTrees
 document'
     = do
       pl <- prolog
@@ -194,7 +223,7 @@ document'
       eof
       return (pl ++ [el] ++ ml)
 
-prolog          :: GenParser Char state XmlTrees
+prolog          :: XParser s XmlTrees
 prolog
     = do
       xml     <- option [] xMLDecl'
@@ -203,7 +232,7 @@ prolog
       misc2   <- many misc
       return (xml ++ misc1 ++ dtdPart ++ misc2)
 
-xMLDecl         :: GenParser Char state XmlTrees
+xMLDecl         :: XParser s XmlTrees
 xMLDecl
     = between (try $ string "<?xml") (string "?>")
       ( do
@@ -215,32 +244,34 @@ xMLDecl
       )
       <?> "xml declaration"
 
-xMLDecl'        :: GenParser Char state XmlTrees
+xMLDecl'        :: XParser s XmlTrees
 xMLDecl'
     = do
       al <- xMLDecl
-      return [mkPi (mkName t_xml) al]
+      return [mkPi' (mkName t_xml) al]
 
-xMLDecl''       :: GenParser Char state XmlTree
+xMLDecl''       :: XParser s XmlTree
 xMLDecl''
     = do
       al     <- option [] (try xMLDecl)
-      return (mkRoot al [])
+      return (mkRoot' al [])
 
-versionInfo     :: GenParser Char state XmlTrees
+versionInfo     :: XParser s XmlTrees
 versionInfo
     = ( do
-        _ <- try ( do
-                   XT.skipS
-                   XT.keyword a_version
-                 )
+        try ( XT.skipS
+              >>
+              XT.keyword a_version
+              >>
+              return ()
+            )
         XT.eq
         vi <- XT.quoted XT.versionNum
-        return [mkAttr (mkName a_version) [mkText vi]]
+        return [mkAttr' (mkName a_version) [mkText' vi]]
       )
       <?> "version info (with quoted version number)"
 
-misc            :: GenParser Char state XmlTree
+misc            :: XParser s XmlTree
 misc
     = comment
       <|>
@@ -248,7 +279,7 @@ misc
       <|>
       ( ( do
           ws <- XT.sPace
-          return (mkText ws)
+          return (mkText' ws)
         ) <?> ""
       )
 
@@ -256,7 +287,7 @@ misc
 --
 -- Document Type definition (2.8)
 
-doctypedecl     :: GenParser Char state XmlTrees
+doctypedecl     :: XParser s XmlTrees
 doctypedecl
     = between (try $ string "<!DOCTYPE") (char '>')
       ( do
@@ -274,10 +305,10 @@ doctypedecl
                     XT.skipS0
                     return m
                   )
-        return [mkDTDElem DOCTYPE ((a_name, n) : exId) markup]
+        return [mkDTDElem' DOCTYPE ((a_name, n) : exId) markup]
       )
 
-markupOrDeclSep :: GenParser Char state XmlTrees
+markupOrDeclSep :: XParser s XmlTrees
 markupOrDeclSep
     = ( do
         ll <- many ( markupdecl
@@ -289,7 +320,7 @@ markupOrDeclSep
         return (concat ll)
       )
 
-declSep         :: GenParser Char state XmlTrees
+declSep         :: XParser s XmlTrees
 declSep
     = XT.mkList XT.peReferenceT
       <|>
@@ -298,7 +329,7 @@ declSep
         return []
       )
 
-markupdecl      :: GenParser Char state XmlTrees
+markupdecl      :: XParser s XmlTrees
 markupdecl
     = XT.mkList
       ( pI
@@ -312,41 +343,44 @@ markupdecl
 --
 -- Standalone Document Declaration (2.9)
 
-sDDecl          :: GenParser Char state XmlTrees
+sDDecl          :: XParser s XmlTrees
 sDDecl
     = do
-      _ <- try (do
-                XT.skipS
-                XT.keyword a_standalone
-               )
+      try ( XT.skipS
+            >>
+            XT.keyword a_standalone
+            >>
+            return ()
+          )
       XT.eq
       sd <- XT.quoted (XT.keywords [v_yes, v_no])
-      return [mkAttr (mkName a_standalone) [mkText sd]]
+      return [mkAttr' (mkName a_standalone) [mkText' sd]]
 
 -- ------------------------------------------------------------
 --
 -- element, tags and content (3, 3.1)
 
-element         :: GenParser Char state XmlTree
+element         :: XParser s XmlTree
 element
+    = char '<'
+      >>
+      element'
+
+element'         :: XParser s XmlTree
+element'
     = ( do
         e <- elementStart
-        elementRest e
+        rwnf e `seq` elementRest e              -- evaluate name and attribute list before parsing contents
       ) <?> "element"
 
-elementStart            :: GenParser Char state (String, [(String, XmlTrees)])
+
+elementStart            :: XParser s (QName, XmlTrees)
 elementStart
     = do
-      n <- ( try ( do
-                   _ <- char '<'
-                   n <- XT.name
-                   return n
-                 )
-             <?> "start tag"
-           )
-      ass <- attrList
+      n  <- XT.name
+      al <- attrList
       XT.skipS0
-      return (n, ass)
+      return (mkName n, al)
       where
       attrList
           = option [] ( do
@@ -357,49 +391,53 @@ elementStart
           = option [] ( do
                         a1 <- attribute
                         al <- attrList
-                        let (n, _v) = a1
-                        if isJust . lookup n $ al
-                          then unexpected ("attribute name " ++ show n ++ " occurs twice in attribute list")
+                        let n = fromJust . getAttrName $ a1
+                        if n `elem` map (fromJust . getAttrName) al
+                          then unexpected
+                               ( "attribute name " ++
+                                 show (qualifiedName n) ++
+                                 " occurs twice in attribute list"
+                               )
                           else return (a1 : al)
                       )
 
-elementRest     :: (String, [(String, XmlTrees)]) -> GenParser Char state XmlTree
+elementRest     :: (QName, XmlTrees) -> XParser s XmlTree
 elementRest (n, al)
     = ( do
-        _ <- try $ string "/>"
-        return $! (mkElement (mkName n) (map (mkA $!) al) [])
+        XT.checkString "/>"
+        return $ mkElement' n al []
       )
       <|>
       ( do
-        _ <- XT.gt
+        XT.gt
         c <- content
         eTag n
-        return $! (mkElement (mkName n) (map (mkA $!) al) $! c)
+        return $ mkElement' n al c
       )
       <?> "proper attribute list followed by \"/>\" or \">\""
-    where
-    mkA (n', ts') = mkAttr (mkName n') ts'
 
-eTag            :: String -> GenParser Char state ()
+eTag            :: QName -> XParser s ()
 eTag n'
     = do
-      _ <- try ( string "</" ) <?> ""
+      XT.checkString "</" <?> ""
       n <- XT.name
       XT.skipS0
-      _ <- XT.gt
-      if n == n'
+      XT.gt
+      if n == qualifiedName n'
          then return ()
-         else unexpected ("illegal end tag </" ++ n ++ "> found, </" ++ n' ++ "> expected")
+         else unexpected ("illegal end tag </" ++ n ++ "> found, </" ++ qualifiedName n' ++ "> expected")
 
-attribute       :: GenParser Char state (String, XmlTrees)
+attribute       :: XParser s XmlTree
 attribute
     = do
       n <- XT.name
       XT.eq
       v <- XT.attrValueT
-      return (n, v)
+      return $ mkAttr' (mkName n) v
 
-content         :: GenParser Char state XmlTrees
+{- this parser corresponds to the XML spec but it's inefficent because of more than 1 char lookahead
+
+content         :: XParser s XmlTrees
 content
     = do
       c1 <- charData
@@ -417,11 +455,45 @@ content
               return (l : c)
             )
       return (c1 ++ concat cl)
+-}
 
-contentWithTextDecl     :: GenParser Char state XmlTrees
+-- this simpler content parser does not need more than a single lookahead
+-- so no try parsers (inefficient) are neccessary
+
+content         :: XParser s XmlTrees
+content
+    = many $
+      ( do		-- parse markup but no closing tags
+        try ( XT.lt
+              >>
+              notFollowedBy (char '/')
+              >>
+              return ()
+            )
+	markup
+      )
+      <|>
+      charData'
+      <|>
+      XT.referenceT
+    where
+    markup
+	= element'
+	  <|>
+	  pI'
+	  <|>
+	  ( char '!'
+            >>
+	    ( comment'
+	      <|>
+	      cDSect'
+            )
+	  )
+
+contentWithTextDecl     :: XParser s XmlTrees
 contentWithTextDecl
-    = do
-      _ <- option [] textDecl
+    = option [] textDecl
+      >>
       content
 
 -- ------------------------------------------------------------
@@ -432,25 +504,25 @@ contentWithTextDecl
 -- first the whole content is detected,
 -- and then, after PE substitution include sections are parsed again
 
-conditionalSect         :: GenParser Char state XmlTree
+conditionalSect         :: XParser s XmlTree
 conditionalSect
     = do
-      _ <- try $ string "<!["
+      XT.checkString "<!["
       cs <- many XD.dtdToken
       _ <- char '['
       sect <- condSectCont
-      return (mkDTDElem CONDSECT [(a_value, sect)] cs)
+      return (mkDTDElem' CONDSECT [(a_value, sect)] cs)
     where
 
-    condSectCont        :: GenParser Char state String
+    condSectCont        :: XParser s String
     condSectCont
-        = ( do
-            _ <- try $ string "]]>"
+        = ( XT.checkString "]]>"
+            >>
             return ""
           )
           <|>
           ( do
-            _ <- try $ string "<!["
+            XT.checkString "<!["
             cs1 <- condSectCont
             cs2 <- condSectCont
             return ("<![" ++ cs1 ++ "]]>" ++ cs2)
@@ -466,7 +538,7 @@ conditionalSect
 --
 -- External Entities (4.2.2)
 
-externalID      :: GenParser Char state Attributes
+externalID      :: XParser s Attributes
 externalID
     = ( do
         _ <- XT.keyword k_system
@@ -490,7 +562,7 @@ externalID
 --
 -- Text Declaration (4.3.1)
 
-textDecl        :: GenParser Char state XmlTrees
+textDecl        :: XParser s XmlTrees
 textDecl
     = between (try $ string "<?xml") (string "?>")
       ( do
@@ -502,26 +574,28 @@ textDecl
       <?> "text declaration"
 
 
-textDecl''      :: GenParser Char state XmlTree
+textDecl''      :: XParser s XmlTree
 textDecl''
     = do
       al    <- option [] (try textDecl)
-      return (mkRoot al [])
+      return (mkRoot' al [])
 
 -- ------------------------------------------------------------
 --
 -- Encoding Declaration (4.3.3)
 
-encodingDecl    :: GenParser Char state XmlTrees
+encodingDecl    :: XParser s XmlTrees
 encodingDecl
     = do
-      _ <- try ( do
-                 XT.skipS
-                 XT.keyword a_encoding
-               )
+      try ( XT.skipS
+            >>
+            XT.keyword a_encoding
+            >>
+            return ()
+          )
       XT.eq
       ed <- XT.quoted XT.encName
-      return [mkAttr (mkName a_encoding) [mkText ed]]
+      return [mkAttr' (mkName a_encoding) [mkText' ed]]
 
 -- ------------------------------------------------------------
 --
@@ -540,7 +614,7 @@ encodingDecl
 
 xread                   :: String -> XmlTrees
 xread str
-    = parseXmlFromString parser loc str
+    = parseXmlFromString parser (withNormNewline ()) loc str
     where
     loc = "string: " ++ show (if length str > 40 then take 40 str ++ "..." else str)
     parser = do
@@ -559,16 +633,16 @@ parseXmlContent
 -- a more general version of 'parseXmlContent'.
 -- The parser to be used and the context are extra parameter
 
-parseXmlText            :: Parser XmlTrees -> String -> XmlTree -> XmlTrees
-parseXmlText p loc      = parseXmlFromString p loc . xshow . (:[])
+parseXmlText            :: SimpleXParser XmlTrees -> XPState () -> String -> XmlTree -> XmlTrees
+parseXmlText p s0 loc   = parseXmlFromString p s0 loc . xshow . (:[])
 
 parseXmlDocument        :: String -> String -> XmlTrees
-parseXmlDocument        = parseXmlFromString document'
+parseXmlDocument        = parseXmlFromString document' (withNormNewline ())
 
-
-parseXmlFromString      :: Parser XmlTrees -> String -> String -> XmlTrees
-parseXmlFromString parser loc
-    = either ((:[]) . mkError c_err . (++ "\n") . show) id . parse parser loc
+parseXmlFromString      :: SimpleXParser XmlTrees -> XPState () -> String -> String -> XmlTrees
+parseXmlFromString parser s0 loc
+    = either ((:[]) . mkError' c_err . (++ "\n") . show) id
+      . runParser parser s0 loc
 
 -- ------------------------------------------------------------
 --
@@ -576,17 +650,17 @@ parseXmlFromString parser loc
 removeEncodingSpec      :: XmlTree -> XmlTrees
 removeEncodingSpec t
     | isText t
-        = ( either ((:[]) . mkError c_err . (++ "\n") . show) ((:[]) . mkText)
-            . parse parser "remove encoding spec"
+        = ( either ((:[]) . mkError' c_err . (++ "\n") . show) ((:[]) . mkText')
+            . runParser parser (withNormNewline ()) "remove encoding spec"
             . fromMaybe ""
             . getText
           ) t
     | otherwise
         = [t]
     where
-    parser :: Parser String
-    parser = do
-             _ <- option [] textDecl
+    parser :: XParser s String
+    parser = option [] textDecl
+             >>
              getInput
 
 -- ------------------------------------------------------------
@@ -594,13 +668,14 @@ removeEncodingSpec t
 -- |
 -- general parser for parsing arbitray parts of a XML document
 
-parseXmlPart    :: Parser XmlTrees -> String -> String -> XmlTree -> XmlTrees
+parseXmlPart    :: SimpleXParser XmlTrees -> String -> String -> XmlTree -> XmlTrees
 parseXmlPart parser expected context t
-    = parseXmlText ( do
-                     res <- parser
-                     eof <?> expected
-                     return res
-                   ) context
+    = parseXmlText
+      ( do
+        res <- parser
+        eof <?> expected
+        return res
+      ) (withoutNormNewline ()) context
       $ t
 
 -- ------------------------------------------------------------
@@ -617,17 +692,17 @@ parseXmlDTDPart
 -- |
 -- Parser for general entites
 
-parseXmlGeneralEntityValue      :: String -> XmlTree -> XmlTrees
-parseXmlGeneralEntityValue
+parseXmlEntityValueAsContent      :: String -> XmlTree -> XmlTrees
+parseXmlEntityValueAsContent
     = parseXmlPart content "general entity value"
 
 -- ------------------------------------------------------------
 
 -- |
--- Parser for attribute values
+-- Parser for entity substitution within attribute values
 
-parseXmlAttrValue       :: String -> XmlTree -> XmlTrees
-parseXmlAttrValue
+parseXmlEntityValueAsAttrValue       :: String -> XmlTree -> XmlTrees
+parseXmlEntityValueAsAttrValue
     = parseXmlPart (XT.attrValueT' "<&") "attribute value"
 
 -- ------------------------------------------------------------
@@ -663,7 +738,7 @@ parseName
 --                        in case of a valid encoding spec
 --                        else the unchanged tree
 
-parseXmlEncodingSpec    :: Parser XmlTree -> XmlTree -> XmlTrees
+parseXmlEncodingSpec    :: SimpleXParser XmlTree -> XmlTree -> XmlTrees
 parseXmlEncodingSpec encDecl x
     = (:[]) .
       ( if isRoot x
@@ -672,14 +747,24 @@ parseXmlEncodingSpec encDecl x
       ) $ x
     where
     parseEncSpec r
-        = case ( parse encDecl source . xshow . getChildren $ r ) of
+        = case ( runParser encDecl (withNormNewline ()) source
+                 . xshow
+                 . getChildren
+                 $ r
+               ) of
           Right t
               -> changeAttrl (mergeAttrl . fromMaybe [] . getAttrl $ t) r
           Left _
               -> r
         where
         -- arrow \"getAttrValue a_source\" programmed on the tree level (oops!)
-        source = xshow . concat . map getChildren . filter ((== a_source) . maybe "" qualifiedName . getAttrName) . fromMaybe [] . getAttrl $ r
+        source = xshow
+                 . concat
+                 . map getChildren
+                 . filter ((== a_source)
+                 . maybe "" qualifiedName . getAttrName)
+                 . fromMaybe []
+                 . getAttrl $ r
 
 parseXmlEntityEncodingSpec      :: XmlTree -> XmlTrees
 parseXmlEntityEncodingSpec      = parseXmlEncodingSpec textDecl''
